@@ -16,34 +16,54 @@ package org.codehaus.plexus.util.xml;
  * limitations under the License.
  */
 
-import org.codehaus.plexus.util.xml.pull.XmlPullParserException;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertNotNull;
+import static org.junit.Assert.assertNotSame;
+import static org.junit.Assert.assertNull;
+import static org.junit.Assert.fail;
 
 import java.io.IOException;
 import java.io.StringReader;
 import java.util.HashMap;
 
-import junit.framework.TestCase;
+import org.codehaus.plexus.util.xml.pull.XmlPullParser;
+import org.codehaus.plexus.util.xml.pull.XmlPullParserException;
+import org.junit.Test;
 
+/**
+ * <p>Xpp3DomTest class.</p>
+ *
+ * @author herve
+ * @version $Id: $Id
+ * @since 3.4.0
+ */
 public class Xpp3DomTest
-    extends TestCase
 {
-
+    /**
+     * <p>testShouldPerformAppendAtFirstSubElementLevel.</p>
+     */
+    @Test
     public void testShouldPerformAppendAtFirstSubElementLevel()
     {
         // create the dominant DOM
         Xpp3Dom t1 = new Xpp3Dom( "top" );
         t1.setAttribute( Xpp3Dom.CHILDREN_COMBINATION_MODE_ATTRIBUTE, Xpp3Dom.CHILDREN_COMBINATION_APPEND );
+        t1.setInputLocation( "t1top" );
 
         Xpp3Dom t1s1 = new Xpp3Dom( "topsub1" );
         t1s1.setValue( "t1s1Value" );
+        t1s1.setInputLocation( "t1s1" );
 
         t1.addChild( t1s1 );
 
         // create the recessive DOM
         Xpp3Dom t2 = new Xpp3Dom( "top" );
+        t2.setInputLocation( "t2top" );
 
         Xpp3Dom t2s1 = new Xpp3Dom( "topsub1" );
         t2s1.setValue( "t2s1Value" );
+        t2s1.setInputLocation( "t2s1" );
 
         t2.addChild( t2s1 );
 
@@ -51,24 +71,38 @@ public class Xpp3DomTest
         Xpp3Dom result = Xpp3Dom.mergeXpp3Dom( t1, t2 );
 
         assertEquals( 2, result.getChildren( "topsub1" ).length );
+        assertEquals( "t2s1Value", result.getChildren( "topsub1" )[0].getValue() );
+        assertEquals( "t1s1Value", result.getChildren( "topsub1" )[1].getValue() );
+
+        assertEquals( "t1top", result.getInputLocation() );
+        assertEquals( "t2s1", result.getChildren( "topsub1" )[0].getInputLocation() );
+        assertEquals( "t1s1", result.getChildren( "topsub1" )[1].getInputLocation() );
     }
 
+    /**
+     * <p>testShouldOverrideAppendAndDeepMerge.</p>
+     */
+    @Test
     public void testShouldOverrideAppendAndDeepMerge()
     {
         // create the dominant DOM
         Xpp3Dom t1 = new Xpp3Dom( "top" );
         t1.setAttribute( Xpp3Dom.CHILDREN_COMBINATION_MODE_ATTRIBUTE, Xpp3Dom.CHILDREN_COMBINATION_APPEND );
+        t1.setInputLocation( "t1top" );
 
         Xpp3Dom t1s1 = new Xpp3Dom( "topsub1" );
         t1s1.setValue( "t1s1Value" );
+        t1s1.setInputLocation( "t1s1" );
 
         t1.addChild( t1s1 );
 
         // create the recessive DOM
         Xpp3Dom t2 = new Xpp3Dom( "top" );
+        t2.setInputLocation( "t2top" );
 
         Xpp3Dom t2s1 = new Xpp3Dom( "topsub1" );
         t2s1.setValue( "t2s1Value" );
+        t2s1.setInputLocation( "t2s1" );
 
         t2.addChild( t2s1 );
 
@@ -76,13 +110,22 @@ public class Xpp3DomTest
         Xpp3Dom result = Xpp3Dom.mergeXpp3Dom( t1, t2, Boolean.TRUE );
 
         assertEquals( 1, result.getChildren( "topsub1" ).length );
+        assertEquals( "t1s1Value", result.getChildren( "topsub1" )[0].getValue() );
+
+        assertEquals( "t1top", result.getInputLocation() );
+        assertEquals( "t1s1", result.getChildren( "topsub1" )[0].getInputLocation() );
     }
 
+    /**
+     * <p>testShouldPerformSelfOverrideAtTopLevel.</p>
+     */
+    @Test
     public void testShouldPerformSelfOverrideAtTopLevel()
     {
         // create the dominant DOM
         Xpp3Dom t1 = new Xpp3Dom( "top" );
         t1.setAttribute( "attr", "value" );
+        t1.setInputLocation( "t1top" );
 
         t1.setAttribute( Xpp3Dom.SELF_COMBINATION_MODE_ATTRIBUTE, Xpp3Dom.SELF_COMBINATION_OVERRIDE );
 
@@ -90,24 +133,32 @@ public class Xpp3DomTest
         Xpp3Dom t2 = new Xpp3Dom( "top" );
         t2.setAttribute( "attr2", "value2" );
         t2.setValue( "t2Value" );
+        t2.setInputLocation( "t2top" );
 
         // merge and check results.
         Xpp3Dom result = Xpp3Dom.mergeXpp3Dom( t1, t2 );
 
         assertEquals( 2, result.getAttributeNames().length );
         assertNull( result.getValue() );
+        assertEquals( "t1top", result.getInputLocation() );
     }
 
+    /**
+     * <p>testShouldMergeValuesAtTopLevelByDefault.</p>
+     */
+    @Test
     public void testShouldMergeValuesAtTopLevelByDefault()
     {
         // create the dominant DOM
         Xpp3Dom t1 = new Xpp3Dom( "top" );
         t1.setAttribute( "attr", "value" );
+        t1.setInputLocation( "t1top" );
 
         // create the recessive DOM
         Xpp3Dom t2 = new Xpp3Dom( "top" );
         t2.setAttribute( "attr2", "value2" );
         t2.setValue( "t2Value" );
+        t2.setInputLocation( "t2top" );
 
         // merge and check results.
         Xpp3Dom result = Xpp3Dom.mergeXpp3Dom( t1, t2 );
@@ -116,8 +167,13 @@ public class Xpp3DomTest
         assertEquals( 2, result.getAttributeNames().length );
 
         assertEquals( result.getValue(), t2.getValue() );
+        assertEquals( "t2top", result.getInputLocation() );
     }
 
+    /**
+     * <p>testShouldMergeValuesAtTopLevel.</p>
+     */
+    @Test
     public void testShouldMergeValuesAtTopLevel()
     {
         // create the dominant DOM
@@ -138,6 +194,10 @@ public class Xpp3DomTest
         assertEquals( result.getValue(), t2.getValue() );
     }
 
+    /**
+     * <p>testNullAttributeNameOrValue.</p>
+     */
+    @Test
     public void testNullAttributeNameOrValue()
     {
         Xpp3Dom t1 = new Xpp3Dom( "top" );
@@ -161,6 +221,10 @@ public class Xpp3DomTest
         t1.toString();
     }
 
+    /**
+     * <p>testEquals.</p>
+     */
+    @Test
     public void testEquals()
     {
         Xpp3Dom dom = new Xpp3Dom( "top" );
@@ -170,6 +234,13 @@ public class Xpp3DomTest
         assertFalse( dom.equals( new Xpp3Dom( (String) null ) ) );
     }
 
+    /**
+     * <p>testEqualsIsNullSafe.</p>
+     *
+     * @throws org.codehaus.plexus.util.xml.pull.XmlPullParserException if any.
+     * @throws java.io.IOException if any.
+     */
+    @Test
     public void testEqualsIsNullSafe()
         throws XmlPullParserException, IOException
     {
@@ -196,14 +267,23 @@ public class Xpp3DomTest
         }
     }
 
+    /**
+     * <p>testShouldOverwritePluginConfigurationSubItemsByDefault.</p>
+     *
+     * @throws org.codehaus.plexus.util.xml.pull.XmlPullParserException if any.
+     * @throws java.io.IOException if any.
+     */
+    @Test
     public void testShouldOverwritePluginConfigurationSubItemsByDefault()
         throws XmlPullParserException, IOException
     {
         String parentConfigStr = "<configuration><items><item>one</item><item>two</item></items></configuration>";
-        Xpp3Dom parentConfig = Xpp3DomBuilder.build( new StringReader( parentConfigStr ) );
+        Xpp3Dom parentConfig =
+            Xpp3DomBuilder.build( new StringReader( parentConfigStr ), new FixedInputLocationBuilder( "parent" ) );
 
         String childConfigStr = "<configuration><items><item>three</item></items></configuration>";
-        Xpp3Dom childConfig = Xpp3DomBuilder.build( new StringReader( childConfigStr ) );
+        Xpp3Dom childConfig =
+            Xpp3DomBuilder.build( new StringReader( childConfigStr ), new FixedInputLocationBuilder( "child" ) );
 
         Xpp3Dom result = Xpp3Dom.mergeXpp3Dom( childConfig, parentConfig );
         Xpp3Dom items = result.getChild( "items" );
@@ -212,16 +292,27 @@ public class Xpp3DomTest
 
         Xpp3Dom item = items.getChild( 0 );
         assertEquals( "three", item.getValue() );
+        assertEquals( "child", item.getInputLocation() );
     }
 
+    /**
+     * <p>testShouldMergePluginConfigurationSubItemsWithMergeAttributeSet.</p>
+     *
+     * @throws org.codehaus.plexus.util.xml.pull.XmlPullParserException if any.
+     * @throws java.io.IOException if any.
+     */
+    @Test
     public void testShouldMergePluginConfigurationSubItemsWithMergeAttributeSet()
         throws XmlPullParserException, IOException
     {
         String parentConfigStr = "<configuration><items><item>one</item><item>two</item></items></configuration>";
-        Xpp3Dom parentConfig = Xpp3DomBuilder.build( new StringReader( parentConfigStr ) );
+        Xpp3Dom parentConfig =
+            Xpp3DomBuilder.build( new StringReader( parentConfigStr ), new FixedInputLocationBuilder( "parent" ) );
 
-        String childConfigStr = "<configuration><items combine.children=\"append\"><item>three</item></items></configuration>";
-        Xpp3Dom childConfig = Xpp3DomBuilder.build( new StringReader( childConfigStr ) );
+        String childConfigStr =
+            "<configuration><items combine.children=\"append\"><item>three</item></items></configuration>";
+        Xpp3Dom childConfig =
+            Xpp3DomBuilder.build( new StringReader( childConfigStr ), new FixedInputLocationBuilder( "child" ) );
 
         Xpp3Dom result = Xpp3Dom.mergeXpp3Dom( childConfig, parentConfig );
         Xpp3Dom items = result.getChild( "items" );
@@ -231,10 +322,19 @@ public class Xpp3DomTest
         Xpp3Dom[] item = items.getChildren();
 
         assertEquals( "one", item[0].getValue() );
+        assertEquals( "parent", item[0].getInputLocation() );
         assertEquals( "two", item[1].getValue() );
+        assertEquals( "parent", item[1].getInputLocation() );
         assertEquals( "three", item[2].getValue() );
+        assertEquals( "child", item[2].getInputLocation() );
     }
 
+    /**
+     * <p>testShouldNotChangeUponMergeWithItselfWhenFirstOrLastSubItemIsEmpty.</p>
+     *
+     * @throws java.lang.Exception if any.
+     */
+    @Test
     public void testShouldNotChangeUponMergeWithItselfWhenFirstOrLastSubItemIsEmpty()
         throws Exception
     {
@@ -252,6 +352,12 @@ public class Xpp3DomTest
         assertEquals( null, items.getChild( 2 ).getValue() );
     }
 
+    /**
+     * <p>testShouldCopyRecessiveChildrenNotPresentInTarget.</p>
+     *
+     * @throws java.lang.Exception if any.
+     */
+    @Test
     public void testShouldCopyRecessiveChildrenNotPresentInTarget()
         throws Exception
     {
@@ -269,4 +375,75 @@ public class Xpp3DomTest
         assertNotSame( result.getChild( "bar" ), recessiveConfig.getChild( "bar" ) );
     }
 
+    /**
+     * <p>testDupeChildren.</p>
+     *
+     * @throws java.io.IOException if any.
+     * @throws org.codehaus.plexus.util.xml.pull.XmlPullParserException if any.
+     */
+    @Test
+    public void testDupeChildren()
+        throws IOException, XmlPullParserException
+    {
+        String dupes = "<configuration><foo>x</foo><foo>y</foo></configuration>";
+        Xpp3Dom dom = Xpp3DomBuilder.build( new StringReader( dupes ) );
+        assertNotNull( dom );
+        assertEquals( "y", dom.getChild( "foo" ).getValue() );
+    }
+
+    /**
+     * <p>testShouldRemoveEntireElementWithAttributesAndChildren.</p>
+     *
+     * @throws java.lang.Exception if any.
+     */
+    @Test
+    public void testShouldRemoveEntireElementWithAttributesAndChildren()
+        throws Exception
+    {
+        String dominantStr = "<config><service combine.self=\"remove\"/></config>";
+        String recessiveStr = "<config><service><parameter>parameter</parameter></service></config>";
+        Xpp3Dom dominantConfig = Xpp3DomBuilder.build( new StringReader( dominantStr ) );
+        Xpp3Dom recessiveConfig = Xpp3DomBuilder.build( new StringReader( recessiveStr ) );
+
+        Xpp3Dom result = Xpp3Dom.mergeXpp3Dom( dominantConfig, recessiveConfig );
+
+        assertEquals( 0, result.getChildCount() );
+        assertEquals( "config", result.getName() );
+    }
+
+    /**
+     * <p>testShouldRemoveDoNotRemoveTagWhenSwappedInputDOMs.</p>
+     *
+     * @throws java.lang.Exception if any.
+     */
+    @Test
+    public void testShouldRemoveDoNotRemoveTagWhenSwappedInputDOMs()
+        throws Exception
+    {
+        String dominantStr = "<config><service combine.self=\"remove\"/></config>";
+        String recessiveStr = "<config><service><parameter>parameter</parameter></service></config>";
+        Xpp3Dom dominantConfig = Xpp3DomBuilder.build( new StringReader( dominantStr ) );
+        Xpp3Dom recessiveConfig = Xpp3DomBuilder.build( new StringReader( recessiveStr ) );
+
+        // same DOMs as testShouldRemoveEntireElementWithAttributesAndChildren(), swapping dominant <--> recessive
+        Xpp3Dom result = Xpp3Dom.mergeXpp3Dom( recessiveConfig, dominantConfig );
+
+        assertEquals( recessiveConfig.toString(), result.toString() );
+    }
+
+    private static class FixedInputLocationBuilder
+        implements Xpp3DomBuilder.InputLocationBuilder
+    {
+        private final Object location;
+
+        public FixedInputLocationBuilder( Object location )
+        {
+            this.location = location;
+        }
+
+        public Object toInputLocation( XmlPullParser parser )
+        {
+            return location;
+        }
+    }
 }
